import bunch


class Message:

    """
    Class responsible for encapsulating
    multiple sub-templates, and when called for,
    getting them all rendered.

    `env` must have a loader that supports an
    `add_templates` contextmanager for defining
    additional templates.

    `sub_templates` must be a dict, where
    the keys are names and the values
    are jinja2 templates.

    """

    REQUIRED_SUB_TEMPLATES = (
        'subject',
        'body',
        'html'
    )

    def __init__(self, env, sub_templates):
        self.env = env
        self.sub_templates = sub_templates

        # Messages must be initialized with all of the
        # REQUIRED_SUB_TEMPLATES defined as keys.
        assert len(self.REQUIRED_SUB_TEMPLATES) == len(list(filter(
            lambda t: t in self.REQUIRED_SUB_TEMPLATES,
            self.sub_templates.keys()
        ))), 'Missing one of the required templates; only have: {}'.format(
            list(self.sub_templates.keys()))

    def render_part(self, name, **context):
        """
        Call this method with a given sub-template's
        name and the context to render it.

        Works by adding our sub-templates to the
        loader's search path just-in-time. This
        has an advantage over `Template.from_string`
        loading in that it supports base templates.

        """
        return self.env.get_template(name).render(**context)

    def render(self, **context):
        """
        Return a `bunch.Bunch` of the initialized
        sub-templates, rendered with the given
        context.

        """
        values = bunch.Bunch()
        with self.env.loader.add_templates(self.sub_templates):
            for name in list(self.sub_templates.keys()):
                values[name] = self.render_part(name, **context)
            return values



import contextlib
import jinja2


class SubTemplateLoader(jinja2.ChoiceLoader):

    """
    A subclass of ChoiceLoader that takes a
    directory where templates can be found,
    as well as providing a contextmanager
    for adding a dict of extra string
    templates.

    """

    def __init__(self, search_path):
        self.search_path = search_path
        super(SubTemplateLoader, self).__init__([
            jinja2.FileSystemLoader(search_path),
            jinja2.DictLoader({}),
        ])

    @property
    def extra(self):
        return self.loaders[1].mapping

    @extra.setter
    def extra(self, extra):
        self.loaders[1].mapping = extra

    @contextlib.contextmanager
    def add_templates(self, templates):
        try:
            self.extra = templates
            yield self
        finally:
            self.extra = {}



import unittest.mock as mock

import simple_mail as mail

from templated_mail.message_loader import MessageLoader
from templated_mail.sub_template_loader import SubTemplateLoader


class TemplatedMail(object):

    def __init__(self, config, logger=None):
        """
        Store config for the mail server,
        and the location of email templates.

        """

        self.config = config
        self.config.logger = logger if logger is not None else mock.Mock()
        self.mail = mail.Mail(config)
        self.loader = MessageLoader(config)

    def send_message(self, name, recipients, context=None):
        """
            1) Build the email template of `name`, as found
            under the configured message directory.
            2) Load and render the subject, text and HTML,
            as defined in <MESSAGE_DIR>/use_my_service.msg,
            with the given context.

        """

        msg = self.loader.get_message(name)
        values = msg.render(**context)
        if msg is not None:
            self.mail.send_message(
                values.subject,
                recipients,
                values.body,
                values.html
            )

        else:
            self.config.logger.error('couldn\'t render a template.')



"""
Package that allows defining email html and metadata
in a template, and sending it by name.

"""

from distutils.core import setup
import setuptools  # this import is needed so that some options and commands work

setup(
    name='templated-mail',
    version='0.2.0.0',
    author='Brian E. Peterson',
    author_email='bepetersondev@gmail.com',
    url='https://github.com/bepetersn/templated-mail',
    zip_safe=False,
    description=__doc__,
    packages=[
        'templated_mail'
    ],
    install_requires=[
    ],
)




import os
import functools

import jinja2
import simple_configparser

from templated_mail.sub_template_loader import SubTemplateLoader
from templated_mail.message import Message


class MessageLoader(object):

    """
    A "loader" (though not a Jinja2 subclass) for
    finding files ending in '.msg' in `config.MESSAGE_DIR`.

    """

    def __init__(self, config):
        self.config = config
        self.search_path = config.MESSAGE_DIR
        self.message_class = functools.partial(Message, jinja2.Environment(
            # Pass the MESSAGE_DIR to sub-template loader
            # so it can find templates for inheritance
            loader=SubTemplateLoader(
                config.MESSAGE_DIR
            )
        ))

    def get_message(self, name):
        """

        :param name: Takes the name of a message in config.MESSAGE_DIR,
                     without the '.msg' extension.
        :return:     The corresponding Message if it could be found,
                     otherwise None.
        """

        try:
            with open(os.path.join(self.search_path, '{}.msg'.format(name))) as f:
                parser = simple_configparser.SimpleConfigParser()
                parser.read_file(f)
                return self.message_class(parser.items())
        except FileNotFoundError:
            self.config.logger.error('couldn\'t find the file')
            return None